import React from 'react';
import toFixed from '../../util/to-fixed';
import { ExpandingResult } from './expanding-result';
import * as Parts from './parts';
var warningIcon = React.createElement("span", null, "\u26A0\uFE0F");
function getDiff(_a) {
    var result = _a.result, pinned = _a.pinned;
    if (!pinned) {
        return 0;
    }
    return toFixed(1 - pinned.averageMs / result.averageMs);
}
function DiffToPinned(_a) {
    var result = _a.result, pinned = _a.pinned;
    if (pinned == null) {
        return null;
    }
    return (React.createElement(React.Fragment, null,
        React.createElement(Parts.Heading, null, "Compared with pinned"),
        React.createElement(Parts.Content, null,
            React.createElement(Parts.Table, null,
                React.createElement("tbody", null,
                    React.createElement("tr", null,
                        React.createElement(Parts.TitleCell, null, "Pinned value"),
                        React.createElement(Parts.ValueCell, null,
                            React.createElement(Parts.ValueLozenge, { type: "raw" },
                                toFixed(pinned.averageMs, 1),
                                "ms"))),
                    React.createElement("tr", null,
                        React.createElement(Parts.TitleCell, null, "Current value"),
                        React.createElement(Parts.ValueCell, null,
                            React.createElement(Parts.ValueLozenge, { type: "raw" },
                                toFixed(result.averageMs, 1),
                                "ms"))),
                    React.createElement("tr", null,
                        React.createElement(Parts.TitleCell, null, "Difference"),
                        React.createElement(Parts.ValueCell, null,
                            React.createElement(DiffLozenge, { diff: getDiff({ result: result, pinned: pinned }) }))))))));
}
function Variance(_a) {
    var result = _a.result;
    if (result.samples === 1) {
        return null;
    }
    var wasStable = result.variance.standardDeviation < 1;
    return (React.createElement(React.Fragment, null,
        React.createElement(Parts.Heading, null, "Variance"),
        React.createElement(Parts.Note, null, "When doing multiple runs the can be differences between the runs. The lower the variance, the higher confidence you can have"),
        React.createElement(Parts.Table, null,
            React.createElement("tbody", null,
                React.createElement("tr", null,
                    React.createElement(Parts.TitleCell, null, "Samples"),
                    React.createElement(Parts.ValueCell, null,
                        React.createElement(Parts.ValueLozenge, { type: "info" }, result.samples))),
                React.createElement("tr", null,
                    React.createElement(Parts.TitleCell, null, "Standard deviation"),
                    React.createElement(Parts.ValueCell, null,
                        React.createElement(Parts.ValueLozenge, { type: wasStable ? 'positive' : 'negative' },
                            wasStable ? null : warningIcon,
                            toFixed(result.variance.standardDeviation)))),
                React.createElement("tr", null,
                    React.createElement(Parts.TitleCell, null, "Lowest"),
                    React.createElement(Parts.ValueCell, null,
                        React.createElement(Parts.ValueLozenge, { type: "raw" },
                            "-",
                            toFixed(result.variance.lowerPercentage),
                            "%"))),
                React.createElement("tr", null,
                    React.createElement(Parts.TitleCell, null, "Highest"),
                    React.createElement(Parts.ValueCell, null,
                        React.createElement(Parts.ValueLozenge, { type: "raw" },
                            "+",
                            toFixed(result.variance.upperPercentage),
                            "%")))))));
}
function Expanded(_a) {
    var task = _a.task, result = _a.result, pinned = _a.pinned;
    return (React.createElement(React.Fragment, null,
        React.createElement(DiffToPinned, { result: result, pinned: pinned }),
        React.createElement(Variance, { result: result }),
        React.createElement(Parts.Heading, null, "Description"),
        React.createElement(Parts.Content, null, task.description)));
}
function DiffLozenge(_a) {
    var diff = _a.diff;
    var type = (function () {
        if (diff > 1) {
            return 'negative';
        }
        if (diff < -1) {
            return 'positive';
        }
        return 'faint';
    })();
    return React.createElement(Parts.ValueLozenge, { type: type },
        diff,
        "%");
}
export default function Timed(_a) {
    var task = _a.task, pinned = _a.pinned, result = _a.result;
    var diff = getDiff({ result: result, pinned: pinned });
    var resultNode = (React.createElement(React.Fragment, null,
        diff ? React.createElement(DiffLozenge, { diff: diff }) : null,
        React.createElement(Parts.ResultValue, null, toFixed(result.averageMs)),
        ' ',
        React.createElement(Parts.ResultScale, null, "ms")));
    return (React.createElement(ExpandingResult, { taskId: task.taskId, name: task.name, result: resultNode, getExpanded: function (_a) {
            var isExpanded = _a.isExpanded;
            return isExpanded ? React.createElement(Expanded, { task: task, result: result, pinned: pinned }) : null;
        } }));
}
