var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { useMachine } from '@xstate/react';
import { useEffect } from 'react';
import eventNames from '../events';
import * as coreEvents from '@storybook/core-events';
import { savePinned, clearPinned, getPinned } from './pinned-storage';
import { bindAll } from '../util/bind-channel-events';
function mergeWithResults(_a) {
    var existing = _a.existing, taskId = _a.taskId, result = _a.result;
    return existing.map(function (groupResult) {
        var _a, _b;
        if (groupResult.static[taskId]) {
            return __assign(__assign({}, groupResult), { static: __assign(__assign({}, groupResult.static), (_a = {}, _a[taskId] = result, _a)) });
        }
        if (groupResult.timed[taskId]) {
            return __assign(__assign({}, groupResult), { timed: __assign(__assign({}, groupResult.timed), (_b = {}, _b[taskId] = result, _b)) });
        }
        return groupResult;
    });
}
export default function usePanelMachine(machine, channel) {
    var _a = useMachine(machine), state = _a[0], send = _a[1], service = _a[2];
    useEffect(function bindChannelEvents() {
        var unsubscribe = bindAll(channel, [
            {
                eventName: coreEvents.STORY_RENDERED,
                fn: function (storyName) {
                    service.send('LOADED', { storyName: storyName, pinned: getPinned(storyName) });
                },
            },
            {
                eventName: coreEvents.STORY_CHANGED,
                fn: function () { return service.send('WAIT'); },
            },
        ]);
        return unsubscribe;
    }, [service, channel]);
    useEffect(function () {
        function finishAll(_a) {
            var results = _a.results;
            service.send('FINISH', { results: results });
        }
        function finishOne(_a) {
            var taskId = _a.taskId, result = _a.result;
            var results = mergeWithResults({
                existing: service.state.context.current.results,
                result: result,
                taskId: taskId,
            });
            service.send('FINISH', { results: results });
        }
        var unbindChannel = bindAll(channel, [
            { eventName: eventNames.FINISH_ALL, fn: finishAll },
            { eventName: eventNames.FINISH_ONE, fn: finishOne },
        ]);
        var unsubscribable = service.subscribe(function next(state, event) {
            if (!state.changed) {
                return;
            }
            if (!event) {
                return;
            }
            var _a = state.context, current = _a.current, storyName = _a.storyName;
            if (event.type === 'PIN') {
                savePinned(storyName, current);
                return;
            }
            if (event.type === 'UNPIN') {
                clearPinned(storyName);
                return;
            }
            var samples = current.samples, copies = current.copies;
            if (state.matches('active.running')) {
                if (event.type === 'START_ALL') {
                    channel.emit(eventNames.START_ALL, {
                        samples: samples,
                        copies: copies,
                    });
                    return;
                }
                if (event.type === 'START_ONE') {
                    channel.emit(eventNames.START_ONE, {
                        taskId: event.taskId,
                        samples: samples,
                        copies: copies,
                    });
                }
            }
        });
        return function unsubscribe() {
            unbindChannel();
            unsubscribable.unsubscribe();
        };
    }, [service, channel]);
    return { state: state, send: send, service: service };
}
