var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import ReactDOM from 'react-dom';
import ReactDOMServer from 'react-dom/server';
import { staticTask, taskGroup, timedTask } from './create';
var render = timedTask({
    name: 'Initial render',
    description: "This task records how long ReactDOM.render() takes with your component",
    run: function (_a) {
        var getElement = _a.getElement, container = _a.container;
        return __awaiter(void 0, void 0, void 0, function () {
            return __generator(this, function (_b) {
                ReactDOM.render(getElement(), container);
                return [2];
            });
        });
    },
});
var hydrate = timedTask({
    name: 'Hydrate',
    description: "\n      This task records how long a ReactDOMServer.hydrate() call\n      takes. If you are server side rendering a React component then html is\n      sent down to the browser. Hydration is the process of React reading through\n      the HTML and building up it's internal virtual model. After hydration React\n      is able to take over the HTML as if it had done the original render on the client.\n  ",
    run: function (_a) {
        var getElement = _a.getElement, controls = _a.controls, container = _a.container;
        return __awaiter(void 0, void 0, void 0, function () {
            var html;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        html = ReactDOMServer.renderToString(getElement());
                        container.innerHTML = html;
                        return [4, controls.time(function mount() {
                                return __awaiter(this, void 0, void 0, function () {
                                    return __generator(this, function (_a) {
                                        ReactDOM.hydrate(getElement(), container);
                                        return [2];
                                    });
                                });
                            })];
                    case 1:
                        _b.sent();
                        return [2];
                }
            });
        });
    },
});
var reRender = timedTask({
    name: 'Re render',
    description: "\n      This task records how long it takes to re-render the component with no prop changes.\n      Note: You can improve this score quickly by using React.memo near the top of your\n      component tree.\n  ",
    run: function (_a) {
        var getElement = _a.getElement, controls = _a.controls, container = _a.container;
        return __awaiter(void 0, void 0, void 0, function () {
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        ReactDOM.render(getElement(), container);
                        return [4, controls.time(function mount() {
                                return __awaiter(this, void 0, void 0, function () {
                                    return __generator(this, function (_a) {
                                        ReactDOM.render(getElement(), container);
                                        return [2];
                                    });
                                });
                            })];
                    case 1:
                        _b.sent();
                        return [2];
                }
            });
        });
    },
});
var domElementCount = staticTask({
    name: 'DOM element count',
    description: "\n    The more DOM element your component creates, the more work the browser needs to do\n  ",
    run: function (_a) {
        var getElement = _a.getElement, container = _a.container;
        return __awaiter(void 0, void 0, void 0, function () {
            var allChildren;
            return __generator(this, function (_b) {
                ReactDOM.render(getElement(), container);
                allChildren = Array.from(container.querySelectorAll('*'));
                return [2, String(allChildren.length)];
            });
        });
    },
});
var group = taskGroup({
    name: 'Client',
    timed: [render, reRender, hydrate],
    static: [domElementCount],
});
export default group;
