var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import ReactDOMServer from 'react-dom/server';
import gzip from 'gzip-js';
import { bytesToKiloBytes } from '../util/convert-bytes-to';
import { timedTask, staticTask, taskGroup } from './create';
var renderToString = timedTask({
    name: 'Render to string',
    description: "This task records how long a ReactDOM.renderToString() call takes",
    run: function (_a) {
        var getElement = _a.getElement;
        return __awaiter(void 0, void 0, void 0, function () {
            return __generator(this, function (_b) {
                ReactDOMServer.renderToString(getElement());
                return [2];
            });
        });
    },
});
var renderToStaticMarkup = timedTask({
    name: 'Render to static markup (cannot be hydrated)',
    description: "\n    This task records how long a ReactDOM.renderToStaticMarkup() call takes.\n    This output cannot be hydrated on the client\n  ",
    run: function (_a) {
        var getElement = _a.getElement;
        return __awaiter(void 0, void 0, void 0, function () {
            return __generator(this, function (_b) {
                ReactDOMServer.renderToStaticMarkup(getElement());
                return [2];
            });
        });
    },
});
var getRawStringSizeInKB = staticTask({
    name: 'String output size',
    description: "\n    The size of the string generated by ReactDOM.renderToString().\n  ",
    scale: 'kb',
    run: function (_a) {
        var getElement = _a.getElement;
        return __awaiter(void 0, void 0, void 0, function () {
            var output, blob;
            return __generator(this, function (_b) {
                output = ReactDOMServer.renderToString(getElement());
                blob = new Blob([output]);
                return [2, String(bytesToKiloBytes(blob.size))];
            });
        });
    },
});
var getGzipStringSizeInKB = staticTask({
    name: 'String output size (gzip)',
    description: "\n    The gzipped size of the string generated by ReactDOM.renderToString().\n  ",
    scale: 'kb',
    run: function (_a) {
        var getElement = _a.getElement;
        return __awaiter(void 0, void 0, void 0, function () {
            var output, bytes;
            return __generator(this, function (_b) {
                output = ReactDOMServer.renderToString(getElement());
                bytes = gzip.zip(output, { level: 9 }).length;
                return [2, String(bytesToKiloBytes(bytes))];
            });
        });
    },
});
var getRawStaticMarkupSizeInKB = staticTask({
    name: 'Static markup output size',
    description: "\n    The size of the string generated by ReactDOM.renderToStaticMarkup().\n  ",
    scale: 'kb',
    run: function (_a) {
        var getElement = _a.getElement;
        return __awaiter(void 0, void 0, void 0, function () {
            var output, blob;
            return __generator(this, function (_b) {
                output = ReactDOMServer.renderToStaticMarkup(getElement());
                blob = new Blob([output]);
                return [2, String(bytesToKiloBytes(blob.size))];
            });
        });
    },
});
var getGzipStaticMarkupSizeInKB = staticTask({
    name: 'Static markup output size (gzip)',
    description: "\n    The gzipped size of the string generated by ReactDOM.renderToStaticMarkup().\n  ",
    scale: 'kb',
    run: function (_a) {
        var getElement = _a.getElement;
        return __awaiter(void 0, void 0, void 0, function () {
            var output, bytes;
            return __generator(this, function (_b) {
                output = ReactDOMServer.renderToStaticMarkup(getElement());
                bytes = gzip.zip(output, { level: 9 }).length;
                return [2, String(bytesToKiloBytes(bytes))];
            });
        });
    },
});
var group = taskGroup({
    name: 'Server',
    timed: [renderToString, renderToStaticMarkup],
    static: [
        getRawStringSizeInKB,
        getGzipStringSizeInKB,
        getRawStaticMarkupSizeInKB,
        getGzipStaticMarkupSizeInKB,
    ],
});
export default group;
